namespace org.me.eventinterfaces;

//Sample app by Brian Long (http://blong.com)

{
  This simple example looks at how events are managed in Android via interfaces
  rather than method pointers, and so also shows the new inline interface
  implementation support: http://wiki.oxygenelanguage.com/en/Inline_Interfaces
}

interface

uses
  java.util,
  android.app,
  android.content,
  android.os,
  android.util,
  android.view,
  android.widget;

type
  MainActivity = public class(Activity, View.OnClickListener)
  private
    method ShowMsg(msg: String);
  public
    method onCreate(savedInstanceState: Bundle); override;
    method onClick(v: View);
    method button2OnClick(v: View);
    method button3OnClick(v: View);
    method button6OnClick(v: View);
  end;

implementation

method MainActivity.onCreate(savedInstanceState: Bundle);
begin
  inherited;

  //Set our view from the "main" layout resource
  ContentView := R.layout.main;

  //To respond to button clicks, you need to call the button's setOnClickListener
  //method, passing an object that implements the interface View.OnClickListener.
  //In Oxygene for Java it is more convenient and common to assign a similar value
  //to the button's OnClickListener implied property

  //Example 1:
  //Since we need an interface implemented we can assign an actual object that
  //implements the interface, either a helper object or, in this case, this activity.
  //Of course for any given object it can only implement the interface once - so this
  //activity will serve button1 through the onClick interface implementation method
  var button1: Button := Button(findViewById(R.id.button1));
  button1.OnClickListener := self;

  //Example 2:
  //We can use an inline interface implementation. This creates an anonymous helper
  //class that implements the specified interface and allows us to assign implementations
  //to as many of the interface methods as we are interested in. The rest will get empty
  //stub implementations. For View.OnClickListener there is only one method.
  //In this case we make use of a method in the activity to act as the onClick
  //implementation
  var button2: Button := Button(findViewById(R.id.button2));
  button2.OnClickListener := new interface View.OnClickListener(
    onClick := @button2OnClick);

  //Example 3:
  //Because View.OnClickListener only has one method we can simplify this
  //using special shorthand to make it look more 'normal' to Delphi eyes
  var button3: Button := Button(findViewById(R.id.button3));
  button3.OnClickListener := @button3OnClick;

  //Example 4:
  //Instead of a regular method we can use an anonymous method,
  //which gains access to any local variables in this method we are in
  var button4: Button := Button(findViewById(R.id.button4));
  button4.OnClickListener := new interface View.OnClickListener(
    onClick := method(v: View)
    begin
      ShowMsg('You pressed Button 4')
    end);      

  //Example 5:
  //We can also use a lambda expression to achieve the same end
  var button5: Button := Button(findViewById(R.id.button5));
  button5.OnClickListener := new interface View.OnClickListener(
    onClick := -> ShowMsg('You pressed Button 5'));

  //Example 6:
  //In the case of onClick, this can be set up in the XML layout
  //meaning no setup is required here at all
  //var button6: Button := Button(findViewById(R.id.button6));
end;

method MainActivity.ShowMsg(msg: String);
begin
  Toast.makeText(self, msg, Toast.LENGTH_SHORT).show
end;

method MainActivity.onClick(v: View);
begin
  ShowMsg('You pressed Button 1')
end;

method MainActivity.button2OnClick(v: View);
begin
  ShowMsg('You pressed Button 2')
end;

method MainActivity.button3OnClick(v: View);
begin
  ShowMsg('You pressed Button 3')
end;

method MainActivity.button6OnClick(v: View);
begin
  ShowMsg('You pressed Button 6')
end;

end.